from destipy.utils.http_method import HTTPMethod
from destipy.utils.requester import Requester


class Fireteam:
    """Fireteam endpoints."""
    def __init__(self, requester, logger):
        self.requester: Requester= requester
        self.logger = logger
        self.FIRETEAM_URL = "https://www.bungie.net/Platform/Fireteam/"

    async def GetActivePrivateClanFireteamCount(self, token: dict, group_id: int) -> dict:
        """Gets a count of all active non-public fireteams for the specified clan.
        Maximum value returned is 25.

        Args:
            token (dict): The token to use for authentication
            group_id (int): The group id of the clan.

        Returns:
            dict: The count of all active non-public fireteams for the specified clan.
        """
        try:
            self.logger.info("Getting active non-public fireteam count for {} for clan {}...".format(token["membership_id"], group_id))
            url = self.FIRETEAM_URL + "Clan/{}/ActiveCount/".format(group_id)
            return await self.requester.request(method=HTTPMethod.GET, url=url, access_token=token["access_token"])
        except Exception as ex:
            self.logger.exception(ex)

    async def GetAvailableClanFireteams(
        self,
        activity_type: int,
        date_range: int,
        group_id: int,
        page: int,
        platform: int,
        public_only: int,
        slot_filter: int,
        lang_filter: str = ""
    ) -> dict:
        """Gets a listing of all of this clan's fireteams that are have available slots.
        Caller is not checked for join criteria so caching is maximized.

        Args:
            activity_type (int): The activity type to filter by.
            date_range (int): The date range to grab available fireteams.
            group_id (int): The group id of the clan.
            page (int): Zero based page
            platform (int): The platform filter.
            public_only (int): Determines public/private filtering.
            slot_filter (int): Filters based on available slots
            lang_filter (str): An optional language filter.

        Returns:
            dict: The available clan fireteams.
        """
        try:
            self.logger.info("Getting available clan fireteams for clan {}...".format(group_id))
            url = self.FIRETEAM_URL + "Clan/{}/Available/{}/{}/{}/{}/{}/{}/?lang_filter={}".format(group_id, platform, activity_type, date_range, slot_filter, public_only, page, lang_filter)
            return await self.requester.request(method=HTTPMethod.GET, url=url)
        except Exception as ex:
            self.logger.exception(ex)

    async def SearchPublicAvailableClanFireteams(
        self,
        token: dict,
        activity_type: int,
        date_range: int,
        page: int,
        platform: int,
        slot_filter: int,
        lang_filter: str = ""
    ) -> dict:
        """Gets a listing of all public fireteams starting now with open slots.
        Caller is not checked for join criteria so caching is maximized.

        Args:
            token (dict): The token to use for authentication
            activity_type (int): The activity type to filter by.
            date_range (int): The date range to grab available fireteams.
            page (int): Zero based page
            platform (int): The platform filter.
            slot_filter (int): Filters based on available slots
            lang_filter (str): An optional language filter.

        Raises:
            Exception: Error getting available fireteams. Reason: response

        Returns:
            dict: The available public fireteams.
        """
        try:
            self.logger.info("Getting available clan fireteams...")
            url = self.FIRETEAM_URL + "Search/Available/{}/{}/{}/{}/?lang_filter={}"
            url = url.format(platform, activity_type, date_range, slot_filter, page, lang_filter)
            return await self.requester.request(method=HTTPMethod.GET, url=url, access_token=token["access_token"])
        except Exception as ex:
            self.logger.exception(ex)

    async def GetMyClanFireteams(
        self,
        token: dict,
        group_id: int,
        include_closed: bool,
        page: int,
        platform: int,
        group_filter: bool,
        lang_filter: str = ""
    ) -> dict:
        """Gets a listing of all fireteams that caller is an applicant, a member, or an alternate of.

        Args:
            token (dict): The token to use for authentication
            group_id (int): The group id of the clan. (This parameter is ignored unless the optional query parameter group_filter is true).
            include_closed (bool): If true, return fireteams that have been closed.
            page (int): Deprecated parameter, ignored.
            platform (int): The platform filter.
            group_filter (bool): If true, filter by clan. Otherwise, ignore the clan and show all of the user's fireteams.
            lang_filter (str, optional): An optional language filter.

        Returns:
            dict: The fireteams that caller is an applicant, a member, or an alternate of.
        """
        bool_dict = {True: "true", False: "false"}
        try:
            self.logger.info("Getting own clan fireteams for {} for clan {}...".format(token["membership_id"], group_id))
            url = self.FIRETEAM_URL + "Clan/{}/My/{}/{}/{}/?lang_filter={}&group_filter={}"
            url = url.format(group_id, platform, include_closed, page, lang_filter, bool_dict[group_filter])
            return await self.requester.request(method=HTTPMethod.GET, url=url)
        except Exception as ex:
            self.logger.exception(ex)

    async def GetClanFireteam(
        self,
        token: dict,
        fireteam_id: int,
        group_id: int
    ) -> dict:
        """Gets a specific fireteam.

        Args:
            token (dict): The token to use for authentication
            fireteam_id (int): The unique id of the fireteam.
            group_id (int): The group id of the clan.

        Returns:
            dict: The specific fireteam.
        """
        try:
            self.logger.info("Getting fireteam {} for clan {}...".format(fireteam_id, group_id))
            url = self.FIRETEAM_URL + "Clan/{}/Summary/{}/".format(group_id, fireteam_id)
            return await self.requester.request(method=HTTPMethod.GET, url=url, access_token=token["access_token"])
        except Exception as ex:
            self.logger.exception(ex)



def format_param_form_list(content: list, paramName: str):
    """Formats a list of items into a string for use in a request.

    Args:
        list (list): The list of items to be formatted.
        paramName (str): The name of the parameter.

    Returns:
        str: The formatted list as a string.
    """
    params = paramName + "=" + ",".join(content)
    return params



import logging
import logging.handlers

from .endpoints.app import App
from .endpoints.base import Base
from .endpoints.community_content import CommunityContent
from .endpoints.content import Content
from .endpoints.destiny2 import Destiny2
from .endpoints.fireteam import Fireteam
from .endpoints.forum import Forum
from .endpoints.group_v2 import GroupV2
from .endpoints.social import Social
from .endpoints.tokens import Tokens
from .endpoints.trending import Trending
from .endpoints.user import User
from .manifest import Manifest
from .oauth import OAuth
from .utils.requester import Requester


class DestinyClient():
    """This class is used to setup a client to interact with the Bungie API.
    To access the endpoints, use the given categories given in the documentation as attributes.

    Example:

        client = DestinyClient(<api_key>)

        user = await client.user.GetBungieNetUserById(<membership_id>)

        or

        client = DestinyClient(<api_key>)

        userEndpoints = client.user

        user = await userEndpoints.GetBungieNetUserById(<membership_id>)

    Args:
        api_key (str): The API key to use for authentication
        client_id (str, optional): The client ID to use for OAuth authentication. Defaults to "".
        client_secret (str, optional): The client secret to use for OAuth authentication. Defaults to "".
        redirect_uri (str, optional): The redirect URI to use for OAuth authentication. Defaults to "".
        max_retries (int, optional): The maximum number of retries to make when a request fails. Defaults to 3.
        max_ratelimit_retries (int, optional): The maximum number of retries to make when a request fails due to rate limiting. Defaults to 3.
        log_file (str, optional): The file to log to. Defaults to "logs/destipy.log".
        logger (optional): The logger to use. If none is given, a default logger with a TimedRotatingFileHandler wih backupCount of 7 is used.
        session (aiohttp.ClientSession, optional): The session to use for requests. If none is given, a new session is created. Defaults to None.
    """
    def __init__(
        self, api_key: str,
        client_id: str = "",
        client_secret: str = "",
        redirect_uri: str = "",
        max_ratelimit_retries: int = 3,
        log_file: str = "logs/destipy.log",
        logger = None,
    ) -> None:

        default_logger = logging.getLogger("Destipy")
        default_logger.setLevel(logging.DEBUG)
        formatter = logging.Formatter(fmt="%(asctime)s - %(levelname)-8s - %(name)s - %(message)s", datefmt="%H:%M:%S")
        file_handler = logging.handlers.TimedRotatingFileHandler(
            filename= log_file,
            when="midnight",
            backupCount=7
        )
        file_handler.setFormatter(formatter)
        if default_logger.hasHandlers():
            default_logger.handlers.clear()
        default_logger.addHandler(file_handler)
        self.logger = default_logger if logger is None else logger
        requester = Requester(api_key, max_ratelimit_retries, self.logger)
        self.app: App = App(client_id, requester, self.logger)
        self.base: Base = Base(requester, self.logger)
        self.community_content: CommunityContent = CommunityContent(requester, self.logger)
        self.content: Content = Content(requester, self.logger)
        self.destiny2: Destiny2 = Destiny2(requester, self.logger)
        self.fireteam: Fireteam = Fireteam(requester, self.logger)
        self.forum: Forum = Forum(requester, self.logger)
        self.group_v2: GroupV2 = GroupV2(requester, self.logger)
        self.manifest: Manifest = Manifest(self.destiny2)
        self.oauth: OAuth = OAuth(client_id, client_secret, requester, redirect_uri, self.logger)
        self.social: Social = Social(requester, self.logger)
        self.tokens: Tokens = Tokens(requester, self.logger)
        self.trending: Trending = Trending(requester, self.logger)
        self.user: User = User(requester, self.logger)

    # Source = https://github.com/jgayfer/pydest/blob/master/pydest/pydest.py
    async def decode_hash(self, hash_id, definition, language="en"):
        """Get the corresponding static info for an item given it's hash value from the Manifest
        Args:
            hash_id:
                The unique identifier of the entity to decode
            definition:
                The type of entity to be decoded (ex. 'DestinyClassDefinition')
            language (optional):
                The language to use when retrieving results from the Manifest. Defaults to 'en'
        Returns:
            dict: json corresponding to the given hash_id and definition
        Raises:
            DestipyException
        """
        return await self.manifest.decode_hash(hash_id, definition, language)

    # Source = https://github.com/jgayfer/pydest/blob/master/pydest/pydest.py
    async def update_manifest(self, language='en'):
        """Update the manifest if there is a newer version available
        Args:
            language [optional]:
                The language corresponding to the manifest to update. Defaults to 'en'
        """
        await self.manifest.update_manifest(language)



from destipy.utils.http_method import HTTPMethod
from destipy.utils.requester import Requester


class Content:
    """Content endpoints."""
    def __init__(self, requester, logger):
        self.requester:Requester = requester
        self.logger = logger
        self.CONTENT_URL = "https://www.bungie.net/Platform/Content/"
        self.head_params = {True: "true", False: "false"}

    async def GetContentType(self, content_type: str) -> dict:
        """Gets an object describing a particular variant of content.

        Args:
            content_type (str): The type of content to get.

        Returns:
            dict: The content type.
        """
        try:
            self.logger.info(f"Getting content type for {content_type}...")
            url = self.CONTENT_URL + "GetContentType/{}/".format(content_type)
            return await self.requester.request(method=HTTPMethod.GET, url=url)
        except Exception as ex:
            self.logger.exception(ex)

    async def GetContentById(self, content_id: int, locale: str, head: bool = False) -> dict:
        """Returns a content item referenced by id

        Args:
            content_id (int): The id of the content item
            locale (str): The locale to return the content in
            head (bool, optional): Whether to return the head or not. async defaults to False.

        Returns:
            dict: The content item
        """
        try:
            self.logger.info(f"Getting content by id {content_id}...")
            url = self.CONTENT_URL + "GetContentById/{}/{}/?head={}"
            url = url.format(content_id, locale, self.head_params[head])
            return await self.requester.request(method=HTTPMethod.GET, url=url)
        except Exception as ex:
            self.logger.exception(ex)

    async def GetContentByTagAndType(
        self,
        locale: str,
        tag: str,
        content_type: str,
        head: bool = False
    ) -> dict:
        """Returns the newest item that matches a given tag and Content Type.

        Args:
            locale (str): The locale to return the content in
            tag (str): The tag to search for
            content_type (str): The type of content to search for
            head (bool, optional): Whether to return the head or not. async defaults to False.

        Returns:
            dict: The content item
        """
        try:
            self.logger.info(f"Getting content by tag {tag} and type {content_type}...")
            url = self.CONTENT_URL + "GetContentByTagAndType/{}/{}/{}/?head={}"
            url = url.format(tag, content_type, locale, self.head_params[head])
            return await self.requester.request(method=HTTPMethod.GET, url=url)
        except Exception as ex:
            self.logger.exception(ex)

    async def SearchContentWithText(
        self,
        locale: str,
        search_text: str,
        c_type: str,
        current_page: int,
        source: str,
        tag: str,
        head: bool = False
    ) -> dict:
        """Gets content based on querystring information passed in. Provides basic search and text search capabilities.

        Args:
            locale (str): The locale to return the content in
            search_text (str): The text to search for
            c_type (str): The type of content to search for
            current_page (int): The current page of results
            source (str): The source of the content
            tag (str): The tag to search for
            head (bool, optional): Whether to return the head or not. async defaults to False.

        Returns:
            dict: The content item
        """
        try:
            self.logger.info(f"Searching for content with text {search_text}...")
            query_params = "?head={}&search_text={}&&ctype={}&current_page={}&source={}&tag={}".format(self.head_params[head], search_text, c_type, current_page, source, tag)
            url = self.CONTENT_URL + "SearchContentWithText/{}/{}"
            url = url.format(locale, query_params)
            return await self.requester.request(method=HTTPMethod.GET, url=url)
        except Exception as ex:
            self.logger.exception(ex)

    async def SearchContentByTagAndType(
        self,
        tag: str,
        content_type: str,
        locale: str,
        current_page: int,
        items_per_page: int,
        head: bool = False
    ) -> dict:
        """Searches for Content Items that match the given Tag and Content Type.

        Args:
            tag (str): The tag to search for
            content_type (str): The type of content to search for
            locale (str): The locale to return the content in
            current_page (int): The current page of results
            items_per_page (int): The number of items to return per page
            head (bool, optional): Whether to return the head or not. async defaults to False.

        Returns:
            dict: The content item
        """
        try:
            self.logger.info(f"Searching for content with tag {tag} and type {content_type}...")
            query_params = "?head={}&current_page={}&itemsPerPage={}"
            query_params.format(self.head_params[head], current_page, items_per_page)
            url = self.CONTENT_URL + "SearchContentByTagAndType/{}/{}/{}/{}"
            url = url.format(tag, content_type, locale, query_params)
            return await self.requester.request(method=HTTPMethod.GET, url=url)
        except Exception as ex:
            self.logger.exception(ex)

    async def SearchHelpArticles(self, search_text: str, size: str) -> dict:
        """Search for Help Articles.

        Args:
            search_text (str): The text to search for
            size (str): The size of the results to return

        Returns:
            dict: The help articles
        """
        try:
            self.logger.info(f"Searching for help articles with text {search_text}...")
            url = self.CONTENT_URL + "SearchHelpArticles/{}/{}/".format(search_text, size)
            return await self.requester.request(method=HTTPMethod.GET, url=url)
        except Exception as ex:
            self.logger.exception(ex)

    async def RssNewsArticle(self, page_token: str) -> dict:
        """Returns a JSON string response that is the RSS feed for news articles.

        Args:
            page_token (str): Zero-based pagination token for paging through result sets.

        Returns:
            dict: The RSS feed for news articles
        """
        try:
            self.logger.info(f"Getting RSS news article for {page_token}...")
            url = self.CONTENT_URL + "Rss/NewsArticle/{}/".format(page_token)
            return await self.requester.request(method=HTTPMethod.GET, url=url)
        except Exception as ex:
            self.logger.exception(ex)
